@@ -22,8 +22,6 @@
 #include <config.h>
 #endif
 
-#include <math.h>
-
 #include <glib.h>
 #include <glib/gi18n.h>
 #include <glib-object.h>
@@ -170,7 +168,11 @@ desktop_thumbnailer_class_finalize (DesktopThumbnailerClass *klass)
 static void
 desktop_thumbnailer_finalize(GObject *object)
 {
+  DesktopThumbnailer *thumbnailer = DESKTOP_THUMBNAILER (object);
 
+  g_free (thumbnailer->exec);
+
+  G_OBJECT_CLASS (desktop_thumbnailer_parent_class)->finalize (object);
 }
 
 static void
@@ -192,55 +194,6 @@ te_string_append_quoted (GString     *string,
 
 
 
-static GdkPixbuf *
-desktop_thumbnailer_get_pixbuf (GInputStream *stream,
-                                int dest_width,
-                                int dest_height,
-                                GCancellable *cancellable)
-{
-  GdkPixbuf *source;
-  gdouble    hratio;
-  gdouble    wratio;
-  gint       source_width;
-  gint       source_height;
-  GError    *error = NULL;
-
-  source = gdk_pixbuf_new_from_stream (stream, cancellable, &error);
-
-  if (!source)
-    {
-      g_clear_error (&error);
-      return NULL;
-    }
-
-  /* determine the source pixbuf dimensions */
-  source_width = gdk_pixbuf_get_width (source);
-  source_height = gdk_pixbuf_get_height (source);
-
-
-  /* return the same pixbuf if no scaling is required */
-  if (source_width <= dest_width && source_height <= dest_height)
-    return source;
-
-  /* determine which axis needs to be scaled down more */
-  wratio = (gdouble) source_width / (gdouble) dest_width;
-  hratio = (gdouble) source_height / (gdouble) dest_height;
-
-  /* adjust the other axis */
-  if (hratio > wratio)
-    dest_width = rint (source_width / hratio);
-  else
-    dest_height = rint (source_height / wratio);
-
-  /* scale the pixbuf down to the desired size */
-  return gdk_pixbuf_scale_simple (source,
-                                  MAX (dest_width, 1), MAX (dest_height, 1),
-                                  GDK_INTERP_BILINEAR);
-
-}
-
-
-
 static gboolean
 desktop_thumbnailer_exec_parse (const gchar *exec,
                                 const gchar *file_path,
@@ -305,27 +258,28 @@ desktop_thumbnailer_load_thumbnail (DesktopThumbnailer *thumbnailer,
                                     const gchar        *path,
                                     gint                width,
                                     gint                height,
-                                    GCancellable       *cancellable)
+                                    GCancellable       *cancellable,
+                                    GError            **error)
 {
   GFileIOStream *stream;
   GFile     *tmpfile;
-  gchar     *exec;
+  gchar     *exec, *std_err;
   gchar    **cmd_argv;
-  gchar     *tmpfilepath;
+  const gchar *tmpfilepath;
   gboolean   res;
   gint       cmd_argc;
   gint       size;
   gchar     *working_directory = NULL;
-  GdkPixbuf *pixbuf            = NULL;
-  GError    *error             = NULL;
+  GdkPixbuf *source, *pixbuf = NULL;
+  gboolean verbose;
 
   g_object_get (G_OBJECT (thumbnailer), "exec", &exec, NULL);
 
-  tmpfile = g_file_new_tmp ("tumbler-XXXXXXX.png", &stream, NULL);
+  tmpfile = g_file_new_tmp ("tumbler-XXXXXX.png", &stream, error);
 
   if ( G_LIKELY (tmpfile) )
     {
-      tmpfilepath = g_file_get_path (tmpfile);
+      tmpfilepath = g_file_peek_path (tmpfile);
 
       size = MIN (width, height);
 
@@ -336,28 +290,37 @@ desktop_thumbnailer_load_thumbnail (DesktopThumbnailer *thumbnailer,
                                             tmpfilepath,
                                             &cmd_argc,
                                             &cmd_argv,
-                                            &error);
+                                            error);
 
       if (G_LIKELY (res))
         {
-          if (path != NULL)
-            working_directory = g_path_get_dirname (path);
-
+          working_directory = g_path_get_dirname (path);
 
+          verbose = tumbler_util_is_debug_logging_enabled (G_LOG_DOMAIN);
           res = g_spawn_sync (working_directory,
                               cmd_argv, NULL,
-                              G_SPAWN_SEARCH_PATH,
-                              NULL, NULL,
+                              verbose ? G_SPAWN_SEARCH_PATH
+                                      : G_SPAWN_SEARCH_PATH | G_SPAWN_STDERR_TO_DEV_NULL,
                               NULL, NULL,
+                              NULL, verbose ? &std_err : NULL,
                               NULL,
-                              &error);
+                              error);
+          if (verbose)
+            {
+              g_printerr ("%s", std_err);
+              g_free (std_err);
+            }
 
           if (G_LIKELY (res))
             {
-              pixbuf = desktop_thumbnailer_get_pixbuf (g_io_stream_get_input_stream(G_IO_STREAM(stream)),
-                                                       width,
-                                                       height,
-                                                       cancellable);
+              source = gdk_pixbuf_new_from_stream (g_io_stream_get_input_stream (G_IO_STREAM (stream)),
+                                                   cancellable, error);
+              if (source != NULL)
+                {
+                  pixbuf = tumbler_util_scale_pixbuf (source, width, height);
+                  g_object_unref (source);
+                }
+
               g_unlink (tmpfilepath);
             }
 
@@ -365,11 +328,8 @@ desktop_thumbnailer_load_thumbnail (DesktopThumbnailer *thumbnailer,
           g_strfreev (cmd_argv);
         }
       else
-        {
-          g_warning (_("Malformed command line \"%s\": %s"), exec, error->message);
-          g_clear_error (&error);
-        }
-      g_free (tmpfilepath);
+          g_warning ("Malformed command line \"%s\": %s", exec, (*error)->message);
+
       g_object_unref (tmpfile);
       g_object_unref (stream);
     }
@@ -391,7 +351,6 @@ desktop_thumbnailer_create (TumblerAbstractThumbnailer *thumbnailer,
   TumblerThumbnailFlavor     *flavor;
   TumblerImageData            data;
   const gchar                *uri;
-  gchar                      *path;
   GFile                      *file;
   gint                        height;
   gint                        width;
@@ -407,9 +366,9 @@ desktop_thumbnailer_create (TumblerAbstractThumbnailer *thumbnailer,
     return;
 
   uri = tumbler_file_info_get_uri (info);
-
   file = g_file_new_for_uri (uri);
-  path = g_file_get_path (file);
+
+  g_debug ("'Exec=%s': Handling URI '%s'", DESKTOP_THUMBNAILER (thumbnailer)->exec, uri);
 
   thumbnail = tumbler_file_info_get_thumbnail (info);
   g_assert (thumbnail != NULL);
@@ -419,7 +378,9 @@ desktop_thumbnailer_create (TumblerAbstractThumbnailer *thumbnailer,
 
   tumbler_thumbnail_flavor_get_size (flavor, &width, &height);
 
-  pixbuf = desktop_thumbnailer_load_thumbnail (DESKTOP_THUMBNAILER(thumbnailer), uri, path, width, height, cancellable);
+  pixbuf = desktop_thumbnailer_load_thumbnail (DESKTOP_THUMBNAILER (thumbnailer),
+                                               uri, g_file_peek_path (file),
+                                               width, height, cancellable, &error);
 
   if (pixbuf != NULL)
     {
@@ -440,7 +401,8 @@ desktop_thumbnailer_create (TumblerAbstractThumbnailer *thumbnailer,
 
   if (error != NULL)
     {
-      g_signal_emit_by_name (thumbnailer, "error", uri, error->code, error->message);
+      g_signal_emit_by_name (thumbnailer, "error", uri,
+                             error->domain, error->code, error->message);
       g_error_free (error);
     }
   else
@@ -448,8 +410,6 @@ desktop_thumbnailer_create (TumblerAbstractThumbnailer *thumbnailer,
       g_signal_emit_by_name (thumbnailer, "ready", uri);
     }
 
-  g_free (path);
-
   g_object_unref (thumbnail);
   g_object_unref (flavor);
   g_object_unref (file);
