@@ -26,6 +26,7 @@
 #include <string.h>
 
 #include <glib.h>
+#include <glib/gi18n.h>
 #include <glib/gstdio.h>
 #include <glib/gprintf.h>
 #include <glib-object.h>
@@ -49,7 +50,7 @@ static TumblerThumbnail *xdg_cache_cache_get_thumbnail      (TumblerCache
                                                              TumblerThumbnailFlavor *flavor);
 static void              xdg_cache_cache_cleanup            (TumblerCache           *cache,
                                                              const gchar *const     *base_uris,
-                                                             guint64                 since);
+                                                             gdouble                 since);
 static void              xdg_cache_cache_delete             (TumblerCache           *cache,
                                                              const gchar *const     *uris);
 static void              xdg_cache_cache_copy               (TumblerCache           *cache,
@@ -82,8 +83,8 @@ G_DEFINE_DYNAMIC_TYPE_EXTENDED (XDGCacheCache,
                                 xdg_cache_cache,
                                 G_TYPE_OBJECT,
                                 0,
-                                TUMBLER_ADD_INTERFACE (TUMBLER_TYPE_CACHE,
-                                                       xdg_cache_cache_iface_init));
+                                G_IMPLEMENT_INTERFACE_DYNAMIC (TUMBLER_TYPE_CACHE,
+                                                               xdg_cache_cache_iface_init));
 
 
 
@@ -137,6 +138,12 @@ xdg_cache_cache_init (XDGCacheCache *cache)
 
   flavor = tumbler_thumbnail_flavor_new_large ();
   cache->flavors = g_list_prepend (cache->flavors, flavor);
+
+  flavor = tumbler_thumbnail_flavor_new_x_large ();
+  cache->flavors = g_list_prepend (cache->flavors, flavor);
+
+  flavor = tumbler_thumbnail_flavor_new_xx_large ();
+  cache->flavors = g_list_prepend (cache->flavors, flavor);
 }
 
 
@@ -146,8 +153,9 @@ xdg_cache_cache_finalize (GObject *object)
 {
   XDGCacheCache *cache = XDG_CACHE_CACHE (object);
 
-  g_list_foreach (cache->flavors, (GFunc) g_object_unref, NULL);
-  g_list_free (cache->flavors);
+  g_list_free_full (cache->flavors, g_object_unref);
+
+  G_OBJECT_CLASS (xdg_cache_cache_parent_class)->finalize (object);
 }
 
 
@@ -172,11 +180,11 @@ xdg_cache_cache_get_thumbnail (TumblerCache           *cache,
 static void
 xdg_cache_cache_cleanup (TumblerCache       *cache,
                          const gchar *const *base_uris,
-                         guint64             since)
+                         gdouble             since)
 {
   XDGCacheCache *xdg_cache = XDG_CACHE_CACHE (cache);
   const gchar   *file_basename;
-  guint64        mtime;
+  gdouble        mtime;
   GFile         *base_file;
   GFile         *dummy_file;
   GFile         *original_file;
@@ -273,14 +281,12 @@ xdg_cache_cache_cleanup (TumblerCache       *cache,
             {
               base_file = xdg_cache_cache_get_file (base_uris[n], iter->data);
 
-              filename = g_file_get_path (base_file);
+              filename = (gchar *) g_file_peek_path (base_file);
               if (g_file_test (filename, G_FILE_TEST_IS_REGULAR))
               {
                 g_unlink (filename);
               }
 
-              g_free(filename);
-
               /* releas the base file */
               g_object_unref(base_file);
             }
@@ -321,13 +327,12 @@ xdg_cache_cache_copy_or_move_file (TumblerCache           *cache,
                                    gboolean                do_copy,
                                    const gchar            *from_uri,
                                    const gchar            *to_uri,
-                                   guint64                 mtime)
+                                   gdouble                 mtime)
 {
   GFile    *from_file;
   GFile    *temp_file;
-  gchar    *temp_path;
-  gchar    *dest_path;
-  gchar    *from_path;
+  const gchar *temp_path;
+  const gchar *dest_path;
   gboolean  result;
   GFile    *dest_file;
 
@@ -347,33 +352,28 @@ xdg_cache_cache_copy_or_move_file (TumblerCache           *cache,
 
   if (result)
     {
-      temp_path = g_file_get_path (temp_file);
+      temp_path = g_file_peek_path (temp_file);
 
       if (xdg_cache_cache_write_thumbnail_info (temp_path, to_uri, mtime,
                                                 NULL, NULL))
         {
           dest_file = xdg_cache_cache_get_file (to_uri, flavor);
-          dest_path = g_file_get_path (dest_file);
+          dest_path = g_file_peek_path (dest_file);
 
           if (g_rename (temp_path, dest_path) != 0)
             g_unlink (temp_path);
 
-          g_free (dest_path);
           g_object_unref (dest_file);
         }
       else
         {
           g_unlink (temp_path);
         }
-
-      g_free (temp_path);
     }
   else if (!do_copy)
     {
       /* if the move failed, drop the old cache file */
-      from_path = g_file_get_path (from_file);
-      g_unlink (from_path);
-      g_free (from_path);
+      g_unlink (g_file_peek_path (from_file));
     }
 
   g_object_unref (temp_file);
@@ -390,7 +390,7 @@ xdg_cache_cache_copy_or_move (TumblerCache       *cache,
 {
   XDGCacheCache *xdg_cache = XDG_CACHE_CACHE (cache);
   GFileInfo     *info;
-  guint64        mtime;
+  gdouble        mtime;
   GFile         *dest_source_file;
   GList         *iter;
   guint          n;
@@ -415,8 +415,9 @@ xdg_cache_cache_copy_or_move (TumblerCache       *cache,
         {
           dest_source_file = g_file_new_for_uri (to_uris[n]);
           info = g_file_query_info (dest_source_file,
-                                    G_FILE_ATTRIBUTE_STANDARD_TYPE ","
-                                    G_FILE_ATTRIBUTE_TIME_MODIFIED,
+                                    G_FILE_ATTRIBUTE_STANDARD_TYPE
+                                      "," G_FILE_ATTRIBUTE_TIME_MODIFIED
+                                      "," G_FILE_ATTRIBUTE_TIME_MODIFIED_USEC,
                                     G_FILE_QUERY_INFO_NONE, NULL, NULL);
 
           if (info == NULL)
@@ -490,7 +491,9 @@ xdg_cache_cache_copy_or_move (TumblerCache       *cache,
             }
           else
             {
-              mtime = g_file_info_get_attribute_uint64 (info, G_FILE_ATTRIBUTE_TIME_MODIFIED);
+              mtime = g_file_info_get_attribute_uint64 (info, G_FILE_ATTRIBUTE_TIME_MODIFIED)
+                      + 1e-6 * g_file_info_get_attribute_uint32 (info,
+                                                                 G_FILE_ATTRIBUTE_TIME_MODIFIED_USEC);
               xdg_cache_cache_copy_or_move_file (cache, iter->data, do_copy,
                                                  from_uris[n], to_uris[n],
                                                  mtime);
@@ -537,6 +540,7 @@ xdg_cache_cache_is_thumbnail (TumblerCache *cache,
   GFile         *flavor_dir;
   GFile         *file;
   gchar         *path;
+  gchar         *needle;
 
   g_return_val_if_fail (XDG_CACHE_IS_CACHE (cache), FALSE);
   g_return_val_if_fail (uri != NULL, FALSE);
@@ -559,7 +563,13 @@ xdg_cache_cache_is_thumbnail (TumblerCache *cache,
       g_free (path);
     }
 
-  return is_thumbnail;
+  if (is_thumbnail)
+    return TRUE;
+
+  /* check if it is a thumbnail in a shared repository */
+  needle = g_strrstr (uri, "/.sh_thumbnails/");
+
+  return needle != NULL;
 }
 
 
@@ -649,18 +659,21 @@ xdg_cache_cache_get_temp_file (const gchar            *uri,
 
 
 
+/* Will return %TRUE if the thumbnail was loaded successfully, or did not exist.
+ * Check whether @uri is non-%NULL and @mtime is a valid time to determine
+ * between the two. Will return %FALSE and set @error if the PNG was corrupt. */
 gboolean
 xdg_cache_cache_read_thumbnail_info (const gchar  *filename,
                                      gchar       **uri,
-                                     guint64      *mtime,
+                                     gdouble      *mtime,
                                      GCancellable *cancellable,
                                      GError      **error)
 {
   png_structp png_ptr;
   png_infop   info_ptr;
   png_textp   text_ptr;
-  gboolean    has_uri = FALSE;
-  gboolean    has_mtime = FALSE;
+  gboolean    has_uri;
+  gboolean    has_mtime;
   FILE       *png;
   gint        num_text;
   gint        i;
@@ -698,6 +711,10 @@ xdg_cache_cache_read_thumbnail_info (const gchar  *filename,
                   /* close the PNG file handle */
                   fclose (png);
 
+                  /* set an error */
+                  g_set_error (error, TUMBLER_ERROR, TUMBLER_ERROR_INVALID_FORMAT,
+                               TUMBLER_ERROR_MESSAGE_CORRUPT_THUMBNAIL, filename);
+
                   return FALSE;
                 }
 #endif
@@ -710,7 +727,8 @@ xdg_cache_cache_read_thumbnail_info (const gchar  *filename,
               if (png_get_text (png_ptr, info_ptr, &text_ptr, &num_text) > 0)
                 {
                   /* iterate over all text keys */
-                  for (i = 0; !(has_uri && has_mtime) && i < num_text; ++i)
+                  for (i = 0, has_uri = FALSE, has_mtime = FALSE;
+                       !(has_uri && has_mtime) && i < num_text; ++i)
                     {
                       if (!text_ptr[i].key)
                         continue;
@@ -726,7 +744,7 @@ xdg_cache_cache_read_thumbnail_info (const gchar  *filename,
                           /* remember the Thumb::MTime value */
                           if (text_ptr[i].text != NULL)
                             {
-                              *mtime = atol (text_ptr[i].text);
+                              *mtime = g_ascii_strtod (text_ptr[i].text, NULL);
                               has_mtime = TRUE;
                             }
                         }
@@ -750,7 +768,7 @@ xdg_cache_cache_read_thumbnail_info (const gchar  *filename,
 gboolean
 xdg_cache_cache_write_thumbnail_info (const gchar  *filename,
                                       const gchar  *uri,
-                                      guint64       mtime,
+                                      gdouble       mtime,
                                       GCancellable *cancellable,
                                       GError      **error)
 {
@@ -771,7 +789,7 @@ xdg_cache_cache_write_thumbnail_info (const gchar  *filename,
     {
       if (!g_cancellable_set_error_if_cancelled (cancellable, &err))
         {
-          mtime_str = g_strdup_printf ("%" G_GUINT64_FORMAT, mtime);
+          mtime_str = g_strdup_printf ("%.6f", mtime);
 
           gdk_pixbuf_save (pixbuf, filename, "png", &err,
                            "tEXt::Thumb::URI", uri,
