@@ -25,7 +25,6 @@
 #include <stdlib.h>
 
 #include <glib.h>
-#include <glib/gi18n.h>
 #include <glib/gstdio.h>
 #include <glib-object.h>
 #include <gio/gio.h>
@@ -37,6 +36,8 @@
 #include <xdg-cache/xdg-cache-cache.h>
 #include <xdg-cache/xdg-cache-thumbnail.h>
 
+#include <libxfce4util/libxfce4util.h>
+
 
 
 /* Property identifiers */
@@ -65,10 +66,10 @@ static gboolean xdg_cache_thumbnail_load            (TumblerThumbnail       *thu
                                                      GError                **error);
 static gboolean xdg_cache_thumbnail_needs_update    (TumblerThumbnail       *thumbnail,
                                                      const gchar            *uri,
-                                                     guint64                 mtime);
+                                                     gdouble                 mtime);
 static gboolean xdg_cache_thumbnail_save_image_data (TumblerThumbnail       *thumbnail,
                                                      TumblerImageData       *data,
-                                                     guint64                 mtime,
+                                                     gdouble                 mtime,
                                                      GCancellable           *cancellable,
                                                      GError                **error);
 
@@ -87,7 +88,7 @@ struct _XDGCacheThumbnail
   XDGCacheCache          *cache;
   gchar                  *uri;
   gchar                  *cached_uri;
-  guint64                 cached_mtime;
+  gdouble                 cached_mtime;
 };
 
 
@@ -96,8 +97,8 @@ G_DEFINE_DYNAMIC_TYPE_EXTENDED (XDGCacheThumbnail,
                                 xdg_cache_thumbnail,
                                 G_TYPE_OBJECT,
                                 0,
-                                TUMBLER_ADD_INTERFACE (TUMBLER_TYPE_THUMBNAIL,
-                                                       xdg_cache_thumbnail_thumbnail_init));
+                                G_IMPLEMENT_INTERFACE_DYNAMIC (TUMBLER_TYPE_THUMBNAIL,
+                                                               xdg_cache_thumbnail_thumbnail_init));
 
 
 
@@ -227,7 +228,6 @@ xdg_cache_thumbnail_load (TumblerThumbnail *thumbnail,
   XDGCacheThumbnail *cache_thumbnail = XDG_CACHE_THUMBNAIL (thumbnail);
   GError            *err = NULL;
   GFile             *file;
-  gchar             *path;
 
   g_return_val_if_fail (XDG_CACHE_IS_THUMBNAIL (thumbnail), FALSE);
   g_return_val_if_fail (cancellable == NULL || G_IS_CANCELLABLE (cancellable), FALSE);
@@ -237,20 +237,16 @@ xdg_cache_thumbnail_load (TumblerThumbnail *thumbnail,
 
   file = xdg_cache_cache_get_file (cache_thumbnail->uri, 
                                    cache_thumbnail->flavor);
-  path = g_file_get_path (file);
-  g_object_unref (file);
 
   g_free (cache_thumbnail->cached_uri);
   cache_thumbnail->cached_uri = NULL;
   cache_thumbnail->cached_mtime = 0;
 
-  xdg_cache_cache_read_thumbnail_info (path, 
+  xdg_cache_cache_read_thumbnail_info (g_file_peek_path (file),
                                        &cache_thumbnail->cached_uri,
                                        &cache_thumbnail->cached_mtime,
                                        cancellable, &err);
-
-  /* free the filename */
-  g_free (path);
+  g_object_unref (file);
 
   if (err != NULL)
     {
@@ -265,24 +261,63 @@ xdg_cache_thumbnail_load (TumblerThumbnail *thumbnail,
 
 
 
+static gboolean
+has_valid_shared_thumbnail (const gchar *uri,
+                            const gchar *size,
+                            gdouble      mtime)
+{
+  gchar       *thumbnail_path;
+  gboolean     found;
+
+  thumbnail_path = xfce_create_shared_thumbnail_path (uri, size);
+
+  if (thumbnail_path != NULL && g_file_test (thumbnail_path, G_FILE_TEST_EXISTS))
+    {
+      gdouble        thumb_mtime;
+      gchar         *thumb_uri;
+
+      if (xdg_cache_cache_read_thumbnail_info (thumbnail_path, &thumb_uri, &thumb_mtime, NULL, NULL))
+        found = mtime == thumb_mtime;
+      else
+        found = FALSE;
+    }
+  else
+    found = FALSE;
+
+  /* free memory */
+  g_free (thumbnail_path);
+
+  return found;
+}
+
+
+
 static gboolean
 xdg_cache_thumbnail_needs_update (TumblerThumbnail *thumbnail,
                                   const gchar      *uri,
-                                  guint64           mtime)
+                                  gdouble           mtime)
 {
   XDGCacheThumbnail *cache_thumbnail = XDG_CACHE_THUMBNAIL (thumbnail);
+  gboolean           is_valid        = TRUE;
 
   g_return_val_if_fail (XDG_CACHE_IS_THUMBNAIL (thumbnail), FALSE);
   g_return_val_if_fail (uri != NULL && *uri != '\0', FALSE);
 
-  if (cache_thumbnail->cached_uri == NULL)
-    return TRUE;
+  if (cache_thumbnail->cached_uri == NULL
+    || cache_thumbnail->cached_mtime == 0
+    || strcmp (cache_thumbnail->uri, uri) != 0
+    || cache_thumbnail->cached_mtime != mtime)
+    {
+      is_valid = FALSE;
+    }
 
-  if (cache_thumbnail->cached_mtime == 0)
-    return TRUE;
+  if (!is_valid) /* if the personal repository is invalid, look for a shared thumbnail repository */
+    {
+      if (has_valid_shared_thumbnail (uri, tumbler_thumbnail_flavor_get_name (cache_thumbnail->flavor), mtime))
+        return FALSE;
+    }
 
-  return strcmp (cache_thumbnail->uri, uri) != 0
-    || cache_thumbnail->cached_mtime != mtime;
+  return !is_valid;
 }
 
 
@@ -290,7 +325,7 @@ xdg_cache_thumbnail_needs_update (TumblerThumbnail *thumbnail,
 static gboolean
 xdg_cache_thumbnail_save_image_data (TumblerThumbnail *thumbnail,
                                      TumblerImageData *data,
-                                     guint64           mtime,
+                                     gdouble           mtime,
                                      GCancellable     *cancellable,
                                      GError          **error)
 {
@@ -302,9 +337,8 @@ xdg_cache_thumbnail_save_image_data (TumblerThumbnail *thumbnail,
   GFile             *dest_file;
   GFile             *flavor_dir;
   GFile             *temp_file;
-  gchar             *dest_path;
-  gchar             *flavor_dir_path;
-  gchar             *temp_path;
+  const gchar       *dest_path;
+  const gchar       *temp_path;
   gchar             *mtime_str;
   gint               width;
   gint               height;
@@ -340,25 +374,22 @@ xdg_cache_thumbnail_save_image_data (TumblerThumbnail *thumbnail,
   temp_file = xdg_cache_cache_get_temp_file (cache_thumbnail->uri,
                                              cache_thumbnail->flavor);
   
-  /* determine the flavor directory and its path */
+  /* determine the flavor directory */
   flavor_dir = g_file_get_parent (temp_file);
-  flavor_dir_path = g_file_get_path (flavor_dir);
 
   /* create the flavor directory with user-only read/write/execute permissions */
-  g_mkdir_with_parents (flavor_dir_path, S_IRWXU);
+  g_mkdir_with_parents (g_file_peek_path (flavor_dir), S_IRWXU);
 
-  /* free the flavor dir path and GFile */
-  g_free (flavor_dir_path);
+  /* free the flavor dir GFile */
   g_object_unref (flavor_dir);
 
   /* open a stream to write to (and possibly replace) the temp file */
-  stream = g_file_replace (temp_file, NULL, FALSE, G_FILE_CREATE_NONE, cancellable, 
-                           &err);
+  stream = g_file_replace (temp_file, NULL, FALSE, G_FILE_CREATE_PRIVATE, cancellable, &err);
 
   if (stream != NULL)
     {
       /* convert the modified time of the source URI to a string */
-      mtime_str = g_strdup_printf ("%" G_GUINT64_FORMAT, mtime);
+      mtime_str = g_strdup_printf ("%.6f", mtime);
 
       /* try to save the pixbuf */
       if (gdk_pixbuf_save_to_stream (dest_pixbuf, G_OUTPUT_STREAM (stream), "png",
@@ -372,20 +403,17 @@ xdg_cache_thumbnail_save_image_data (TumblerThumbnail *thumbnail,
                                                 cache_thumbnail->flavor);
 
           /* determine temp and destination paths */
-          temp_path = g_file_get_path (temp_file);
-          dest_path = g_file_get_path (dest_file);
+          temp_path = g_file_peek_path (temp_file);
+          dest_path = g_file_peek_path (dest_file);
 
           /* try to rename the thumbnail */
           if (g_rename (temp_path, dest_path) == -1)
             {
               g_set_error (&err, TUMBLER_ERROR, TUMBLER_ERROR_SAVE_FAILED,
-                           _("Could not save thumbnail to \"%s\""), dest_path);
+                           TUMBLER_ERROR_MESSAGE_SAVE_FAILED, dest_path);
+              g_file_delete (temp_file, NULL, NULL);
             }
 
-          /* free strings */
-          g_free (dest_path);
-          g_free (temp_path);
-
           /* destroy the destination GFile */
           g_object_unref (dest_file);
         }
