@@ -36,7 +36,6 @@
 #include <stdlib.h>
 
 #include <glib.h>
-#include <glib/gi18n.h>
 #include <glib-object.h>
 
 #include <tumbler/tumbler.h>
@@ -62,157 +61,6 @@ shutdown_tumbler (TumblerLifecycleManager *lifecycle_manager,
 
 
 
-static inline gboolean
-xfce_is_valid_tilde_prefix (const gchar *p)
-{
-  if (g_ascii_isspace (*p) /* thunar ~/music */
-      || *p == '=' /* terminal --working-directory=~/ */
-      || *p == '\'' || *p == '"') /* terminal --working-directory '~my music' */
-    return TRUE;
-
-  return FALSE;
-}
-
-
-/* from libxfce4util */
-static gchar *
-xfce_expand_variables (const gchar *command,
-                       gchar      **envp)
-{
-  GString        *buf;
-  const gchar    *start;
-  gchar          *variable;
-  const gchar    *p;
-  const gchar    *value;
-  gchar         **ep;
-  guint           len;
-#ifdef HAVE_GETPWNAM
-  struct passwd  *pw;
-  gchar          *username;
-#endif
-
-  if (G_UNLIKELY (command == NULL))
-    return NULL;
-
-  buf = g_string_sized_new (strlen (command));
-
-  for (p = command; *p != '\0'; ++p)
-    {
-      continue_without_increase:
-
-      if (*p == '~'
-          && (p == command
-              || xfce_is_valid_tilde_prefix (p - 1)))
-        {
-          /* walk to the end of the string or to a directory separator */
-          for (start = ++p; *p != '\0' && *p != G_DIR_SEPARATOR; ++p);
-
-          if (G_LIKELY (start == p))
-            {
-              /* add the current user directory */
-              buf = g_string_append (buf, g_get_home_dir ());
-            }
-          else
-            {
-#ifdef HAVE_GETPWNAM
-              username = g_strndup (start, p - start);
-              pw = getpwnam (username);
-              g_free (username);
-
-              /* add the users' home directory if found, fallback to the
-               * not-expanded string */
-              if (pw != NULL && pw->pw_dir != NULL)
-                buf = g_string_append (buf, pw->pw_dir);
-              else
-#endif
-                buf = g_string_append_len (buf, start - 1, p - start + 1);
-            }
-
-          /* we are either at the end of the string or *p is a separator,
-           * so continue to add it to the result buffer */
-        }
-      else if (*p == '$')
-        {
-          /* walk to the end of a valid variable name */
-          for (start = ++p; *p != '\0' && (g_ascii_isalnum (*p) || *p == '_'); ++p);
-
-          if (start < p)
-            {
-              value = NULL;
-              len = p - start;
-
-              /* lookup the variable in the environment supplied by the user */
-              if (envp != NULL)
-                {
-                  /* format is NAME=VALUE */
-                  for (ep = envp; *ep != NULL; ++ep)
-                    if (strncmp (*ep, start, len) == 0
-                        && (*ep)[len] == '=')
-                      {
-                        value = (*ep) + len + 1;
-                        break;
-                      }
-                }
-
-              /* fallback to the environment */
-              if (value == NULL)
-                {
-                  variable = g_strndup (start, len);
-                  value = g_getenv (variable);
-                  g_free (variable);
-                }
-
-              if (G_LIKELY (value != NULL))
-                {
-                  buf = g_string_append (buf, value);
-                }
-              else
-                {
-                  /* the variable name was valid, but no value was
-                   * found, insert nothing and continue */
-                }
-
-              /* *p is at the start of the charater after the variable,
-               * so continue scanning without advancing the string offset
-               * so two variables are replaced properly */
-              goto continue_without_increase;
-            }
-          else
-            {
-              /* invalid variable format, add the
-               * $ character and continue */
-              --p;
-            }
-        }
-
-      buf = g_string_append_c (buf, *p);
-    }
-
-  return g_string_free (buf, FALSE);
-}
-
-
-
-static GSList *
-locations_from_strv (gchar **array)
-{
-  GSList *locations = NULL;
-  guint   n;
-  gchar  *path;
-
-  if (array == NULL)
-    return NULL;
-
-  for (n = 0; array[n] != NULL; n++)
-    {
-      path = xfce_expand_variables (array[n], NULL);
-      locations = g_slist_prepend (locations, g_file_new_for_commandline_arg (path));
-      g_free (path);
-    }
-
-  return locations;
-}
-
 static void
 on_dbus_name_lost (GDBusConnection *connection,
                    const gchar     *name,
@@ -220,7 +68,7 @@ on_dbus_name_lost (GDBusConnection *connection,
 {
   GMainLoop *main_loop;
 
-  g_critical (_("Name %s lost on the message dbus, exiting."),name);
+  g_critical ("Name %s lost on the message dbus, exiting.",name);
   main_loop = (GMainLoop*)user_data;
   g_main_loop_quit(main_loop);
 }
@@ -256,7 +104,7 @@ main (int    argc,
 
 #ifdef G_OS_UNIX
   if (nice (19) != 19)
-    g_warning (_("Couldn't change nice value of process."));
+    g_warning ("Couldn't change nice value of process.");
 #endif
 
 #ifdef DEBUG
@@ -264,8 +112,16 @@ main (int    argc,
   g_log_set_always_fatal (G_LOG_LEVEL_CRITICAL);
 #endif
 
+#if GLIB_CHECK_VERSION (2, 68, 0)
+  G_GNUC_BEGIN_IGNORE_DEPRECATIONS
+  /* to avoid overlap between stderr and stdout, e.g. when third party APIs write to
+   * stderr, or if debugging macros writing to stderr are used in addition to g_debug() */
+  g_log_writer_default_set_use_stderr (TRUE);
+  G_GNUC_END_IGNORE_DEPRECATIONS
+#endif
+
   /* set the application name. Translators: Don't translate "Tumbler". */
-  g_set_application_name (_("Tumbler Thumbnailing Service"));
+  g_set_application_name ("Tumbler Thumbnailing Service");
 
   connection = g_bus_get_sync (G_BUS_TYPE_SESSION, NULL, &error);
   
@@ -302,46 +158,42 @@ main (int    argc,
       /* add all thumbnailers to the registry */
       for (tp = thumbnailers; tp != NULL; tp = tp->next)
         {
-          /* set settings from rc file */
-          type_name = G_OBJECT_TYPE_NAME (tp->data);
-          priority = g_key_file_get_integer (rc, type_name, "Priority", NULL);
-          file_size = g_key_file_get_int64 (rc, type_name, "MaxFileSize", NULL);
+          /* desktop thumbnailers are set up per desktop file */
+          if (g_object_class_find_property (G_OBJECT_GET_CLASS (tp->data), "exec") == NULL)
+            {
+              /* set settings from rc file */
+              type_name = G_OBJECT_TYPE_NAME (tp->data);
+              priority = g_key_file_get_integer (rc, type_name, "Priority", NULL);
+              file_size = g_key_file_get_int64 (rc, type_name, "MaxFileSize", NULL);
 
-          paths = g_key_file_get_string_list (rc, type_name, "Locations", NULL, NULL);
-          locations = locations_from_strv (paths);
-          g_strfreev (paths);
+              paths = g_key_file_get_string_list (rc, type_name, "Locations", NULL, NULL);
+              locations = tumbler_util_locations_from_strv (paths);
+              g_strfreev (paths);
 
-          paths = g_key_file_get_string_list (rc, type_name, "Excludes", NULL, NULL);
-          excludes = locations_from_strv (paths);
-          g_strfreev (paths);
+              paths = g_key_file_get_string_list (rc, type_name, "Excludes", NULL, NULL);
+              excludes = tumbler_util_locations_from_strv (paths);
+              g_strfreev (paths);
 
-          g_object_set (G_OBJECT (tp->data),
-                        "priority", priority,
-                        "max-file-size", file_size,
-                        "locations", locations,
-                        "excludes", excludes,
-                        NULL);
+              g_object_set (tp->data, "priority", priority, "max-file-size", file_size,
+                            "locations", locations, "excludes", excludes, NULL);
+
+              /* cleanup */
+              g_slist_free_full (locations, g_object_unref);
+              g_slist_free_full (excludes, g_object_unref);
+            }
 
           /* ready for usage */
           tumbler_registry_add (registry, tp->data);
-
-          /* cleanup */
-          g_object_unref (tp->data);
-          g_slist_foreach (locations, (GFunc) g_object_unref, NULL);
-          g_slist_free (locations);
-          g_slist_foreach (excludes, (GFunc) g_object_unref, NULL);
-          g_slist_free (excludes);
         }
 
       /* free the thumbnailer list */
-      g_list_free (thumbnailers);
+      g_list_free_full (thumbnailers, g_object_unref);
     }
 
   g_key_file_free (rc);
 
   /* release all providers and free the provider list */
-  g_list_foreach (providers, (GFunc) g_object_unref, NULL);
-  g_list_free (providers);
+  g_list_free_full (providers, g_object_unref);
 
   /* drop the reference on the provider factory */
   g_object_unref (provider_factory);
@@ -361,7 +213,7 @@ main (int    argc,
   /* try to load specialized thumbnailers and exit if that fails */
   if (!tumbler_registry_load (registry, &error))
     {
-      g_warning (_("Failed to load specialized thumbnailers into the registry: %s"),
+      g_warning ("Failed to load specialized thumbnailers into the registry: %s",
                  error->message);
       g_error_free (error);
 
@@ -417,6 +269,8 @@ main (int    argc,
       /* start the lifecycle manager */
       tumbler_lifecycle_manager_start (lifecycle_manager);
 
+      g_debug ("Ready to handle requests\n");
+
       /* enter the main loop, thereby making the tumbler service available */
       g_main_loop_run (main_loop);
     }
