@@ -75,6 +75,7 @@ static void tumbler_group_scheduler_thread            (gpointer
                                                        gpointer                   user_data);
 static void tumbler_group_scheduler_thumbnailer_error (TumblerThumbnailer        *thumbnailer,
                                                        const gchar               *failed_uri,
+                                                       GQuark                     error_domain,
                                                        gint                       error_code,
                                                        const gchar               *message,
                                                        GList                    **uri_errors);
@@ -105,6 +106,7 @@ struct _TumblerGroupScheduler
 struct _UriError
 {
   guint  error_code;
+  GQuark error_domain;
   gchar *message;
   gchar *failed_uri;
 };
@@ -178,11 +180,8 @@ tumbler_group_scheduler_finalize (GObject *object)
   /* destroy both thread pools */
   g_thread_pool_free (scheduler->pool, TRUE, TRUE);
 
-  /* release all pending requests */
-  g_list_foreach (scheduler->requests, (GFunc) tumbler_scheduler_request_free, NULL);
-
-  /* destroy the request list */
-  g_list_free (scheduler->requests);
+  /* release all pending requests and destroy the request list */
+  g_list_free_full (scheduler->requests, tumbler_scheduler_request_free);
 
   /* free the scheduler name */
   g_free (scheduler->name);
@@ -376,12 +375,14 @@ tumbler_group_scheduler_dequeue_request (TumblerSchedulerRequest *request,
 
 static UriError *
 uri_error_new (gint         code,
+               GQuark       domain,
                const gchar *uri,
                const gchar *message)
 {
   UriError *error;
 
   error = g_slice_new0 (UriError);
+  error->error_domain = domain;
   error->error_code = code;
   error->failed_uri = g_strdup (uri);
   error->message = g_strdup (message);
@@ -392,8 +393,10 @@ uri_error_new (gint         code,
 
 
 static void
-uri_error_free (UriError *error)
+uri_error_free (gpointer data)
 {
+  UriError *error = data;
+
   g_free (error->message);
   g_free (error->failed_uri);
 
@@ -420,9 +423,10 @@ tumbler_group_scheduler_thread (gpointer data,
   GList                   *ready_uris;
   GList                   *cached_uris = NULL;
   GList                   *missing_uris = NULL;
-  GList                   *lp;
+  GList                   *lp, *lq;
   guint                    n;
   gint                     error_code = 0;
+  GQuark                   error_domain = 0;
 
   g_return_if_fail (TUMBLER_IS_GROUP_SCHEDULER (scheduler));
   g_return_if_fail (request != NULL);
@@ -483,7 +487,7 @@ tumbler_group_scheduler_thread (gpointer data,
             {
               /* no thumbnailer for this URI, we need to emit an error */
               g_set_error (&error, TUMBLER_ERROR, TUMBLER_ERROR_UNSUPPORTED,
-                           _("No thumbnailer available for \"%s\""), 
+                           TUMBLER_ERROR_MESSAGE_NO_THUMBNAILER,
                            tumbler_file_info_get_uri (request->infos[n]));
             }
         }
@@ -546,25 +550,33 @@ tumbler_group_scheduler_thread (gpointer data,
         }
       tumbler_mutex_unlock (scheduler->mutex);
 
-      /* connect to the error signal of the thumbnailer */
-      g_signal_connect (request->thumbnailers[n], "error", 
-                        G_CALLBACK (tumbler_group_scheduler_thumbnailer_error),
-                        &uri_errors);
+      for (lq = request->thumbnailers[n]; lq != NULL; lq = lq->next)
+        {
+          /* forward only the error signal of the last thumbnailer */
+          if (lq->next == NULL)
+            g_signal_connect (lq->data, "error",
+                              G_CALLBACK (tumbler_group_scheduler_thumbnailer_error), &uri_errors);
+          else if (tumbler_util_is_debug_logging_enabled (G_LOG_DOMAIN))
+            g_signal_connect (lq->data, "error",
+                              G_CALLBACK (tumbler_scheduler_thumberr_debuglog), request);
 
-      /* connect to the ready signal of the thumbnailer */
-      g_signal_connect (request->thumbnailers[n], "ready",
-                        G_CALLBACK (tumbler_group_scheduler_thumbnailer_ready),
-                        &ready_uris);
+          /* connect to the ready signal of the thumbnailer */
+          g_signal_connect (lq->data, "ready",
+                            G_CALLBACK (tumbler_group_scheduler_thumbnailer_ready), &ready_uris);
 
-      /* tell the thumbnailer to generate the thumbnail */
-      tumbler_thumbnailer_create (request->thumbnailers[n], 
-                                  request->cancellables[n],
-                                  request->infos[n]);
+          /* cancel lower priority thumbnailers for this uri on ready signal */
+          g_signal_connect_swapped (lq->data, "ready", G_CALLBACK (g_cancellable_cancel),
+                                    request->cancellables[n]);
 
-      /* disconnect from all signals when we're finished */
-      g_signal_handlers_disconnect_matched (request->thumbnailers[n],
-                                            G_SIGNAL_MATCH_DATA,
-                                            0, 0, NULL, NULL, request);
+          /* tell the thumbnailer to generate the thumbnail */
+          tumbler_thumbnailer_create (lq->data, request->cancellables[n], request->infos[n]);
+
+          /* disconnect from all signals when we're finished */
+          g_signal_handlers_disconnect_by_data (lq->data, &uri_errors);
+          g_signal_handlers_disconnect_by_data (lq->data, &ready_uris);
+          g_signal_handlers_disconnect_by_data (lq->data, request);
+          g_signal_handlers_disconnect_by_data (lq->data, request->cancellables[n]);
+        }
     }
 
   tumbler_mutex_lock (scheduler->mutex);
@@ -586,8 +598,11 @@ tumbler_group_scheduler_thread (gpointer data,
           uri_error = iter->data;
 
           /* we use the error code of the first failed URI */
-          if (iter == uri_errors) 
-            error_code = uri_error->error_code;
+          if (iter == uri_errors)
+            {
+              error_domain = uri_error->error_domain;
+              error_code = uri_error->error_code;
+            }
 
           if (uri_error->message != NULL)
             {
@@ -608,7 +623,7 @@ tumbler_group_scheduler_thread (gpointer data,
 
       /* forward the error signal */
       g_signal_emit_by_name (request->scheduler, "error", request->handle, 
-                             failed_uris, error_code, message->str, 
+                             failed_uris, error_domain, error_code, message->str,
                              request->origin);
 
       /* free the failed URIs array. Its contents are owned by the URI errors */
@@ -619,8 +634,7 @@ tumbler_group_scheduler_thread (gpointer data,
     }
 
   /* free all URI errors and the error URI list */
-  g_list_foreach (uri_errors, (GFunc) uri_error_free, NULL);
-  g_list_free (uri_errors);
+  g_list_free_full (uri_errors, uri_error_free);
 
   /* check if we have any successfully processed URIs */
   if (ready_uris != NULL)
@@ -644,8 +658,7 @@ tumbler_group_scheduler_thread (gpointer data,
     }
 
   /* free the ready URIs */
-  g_list_foreach (ready_uris, (GFunc) g_free, NULL);
-  g_list_free (ready_uris); 
+  g_list_free_full (ready_uris, g_free);
 
   /* notify others that we're finished processing the request */
   tumbler_group_scheduler_finish_request (scheduler, request);
@@ -658,6 +671,7 @@ tumbler_group_scheduler_thread (gpointer data,
 static void
 tumbler_group_scheduler_thumbnailer_error (TumblerThumbnailer *thumbnailer,
                                            const gchar        *failed_uri,
+                                           GQuark              error_domain,
                                            gint                error_code,
                                            const gchar        *message,
                                            GList             **uri_errors)
@@ -670,7 +684,7 @@ tumbler_group_scheduler_thumbnailer_error (TumblerThumbnailer *thumbnailer,
   g_return_if_fail (uri_errors != NULL);
 
   /* allocate a new URI error */
-  error = uri_error_new (error_code, failed_uri, message);
+  error = uri_error_new (error_code, error_domain, failed_uri, message);
 
   /* add the error to the list */
   *uri_errors = g_list_prepend (*uri_errors, error);
