@@ -60,7 +60,7 @@ static void tumbler_lifo_scheduler_set_property      (GObject
 static void tumbler_lifo_scheduler_push              (TumblerScheduler          *scheduler,
                                                       TumblerSchedulerRequest   *request);
 static void tumbler_lifo_scheduler_dequeue           (TumblerScheduler          *scheduler,
-                                                      guint                      handle);
+                                                      guint32                    handle);
 static void tumbler_lifo_scheduler_cancel_by_mount   (TumblerScheduler          *scheduler,
                                                       GMount                    *mount);
 static void tumbler_lifo_scheduler_finish_request    (TumblerLifoScheduler *scheduler,
@@ -71,6 +71,7 @@ static void tumbler_lifo_scheduler_thread            (gpointer
                                                       gpointer                   user_data);
 static void tumbler_lifo_scheduler_thumbnailer_error (TumblerThumbnailer        *thumbnailer,
                                                       const gchar               *failed_uri,
+                                                      GQuark                     error_domain,
                                                       gint                       error_code,
                                                       const gchar               *message,
                                                       TumblerSchedulerRequest   *request);
@@ -158,11 +159,8 @@ tumbler_lifo_scheduler_finalize (GObject *object)
   /* destroy both thread pools */
   g_thread_pool_free (scheduler->pool, TRUE, TRUE);
 
-  /* release all pending requests */
-  g_list_foreach (scheduler->requests, (GFunc) tumbler_scheduler_request_free, NULL);
-
-  /* destroy the request list */
-  g_list_free (scheduler->requests);
+  /* release all pending requests and destroy the request list */
+  g_list_free_full (scheduler->requests, tumbler_scheduler_request_free);
 
   /* free the scheduler name */
   g_free (scheduler->name);
@@ -244,7 +242,7 @@ tumbler_lifo_scheduler_push (TumblerScheduler        *scheduler,
 
 static void
 tumbler_lifo_scheduler_dequeue (TumblerScheduler *scheduler,
-                                guint             handle)
+                                guint32           handle)
 {
   TumblerLifoScheduler *lifo_scheduler = TUMBLER_LIFO_SCHEDULER (scheduler);
 
@@ -364,7 +362,7 @@ tumbler_lifo_scheduler_thread (gpointer data,
   GError                  *error = NULL;
   GList                   *cached_uris = NULL;
   GList                   *missing_uris = NULL;
-  GList                   *lp;
+  GList                   *lp, *lq;
   guint                    n;
 
   g_return_if_fail (TUMBLER_IS_LIFO_SCHEDULER (scheduler));
@@ -419,7 +417,7 @@ tumbler_lifo_scheduler_thread (gpointer data,
             {
               /* no thumbnailer for this URI, we need to emit an error */
               g_set_error (&error, TUMBLER_ERROR, TUMBLER_ERROR_UNSUPPORTED,
-                           _("No thumbnailer available for \"%s\""), 
+                           TUMBLER_ERROR_MESSAGE_NO_THUMBNAILER,
                            tumbler_file_info_get_uri (request->infos[n]));
             }
         }
@@ -477,29 +475,35 @@ tumbler_lifo_scheduler_thread (gpointer data,
           return;
         }
 
-      /* We immediately forward error and ready so that clients rapidly know
-       * when individual thumbnails are ready. It's a LIFO for better inter-
-       * activity with the clients, so we assume this behaviour to be desired. */
+      for (lq = request->thumbnailers[n]; lq != NULL; lq = lq->next)
+        {
+          /* We immediately forward error and ready so that clients rapidly know
+           * when individual thumbnails are ready. It's a LIFO for better inter-
+           * activity with the clients, so we assume this behaviour to be desired. */
 
-      /* connect to the error signal of the thumbnailer */
-      g_signal_connect (request->thumbnailers[n], "error", 
-                        G_CALLBACK (tumbler_lifo_scheduler_thumbnailer_error),
-                        request);
+          /* forward only the error signal of the last thumbnailer */
+          if (lq->next == NULL)
+            g_signal_connect (lq->data, "error",
+                              G_CALLBACK (tumbler_lifo_scheduler_thumbnailer_error), request);
+          else if (tumbler_util_is_debug_logging_enabled (G_LOG_DOMAIN))
+            g_signal_connect (lq->data, "error",
+                              G_CALLBACK (tumbler_scheduler_thumberr_debuglog), request);
 
-      /* connect to the ready signal of the thumbnailer */
-      g_signal_connect (request->thumbnailers[n], "ready",
-                        G_CALLBACK (tumbler_lifo_scheduler_thumbnailer_ready),
-                        request);
+          /* connect to the ready signal of the thumbnailer */
+          g_signal_connect (lq->data, "ready",
+                            G_CALLBACK (tumbler_lifo_scheduler_thumbnailer_ready), request);
 
-      /* tell the thumbnailer to generate the thumbnail */
-      tumbler_thumbnailer_create (request->thumbnailers[n], 
-                                  request->cancellables[n],
-                                  request->infos[n]);
+          /* cancel lower priority thumbnailers for this uri on ready signal */
+          g_signal_connect_swapped (lq->data, "ready", G_CALLBACK (g_cancellable_cancel),
+                                    request->cancellables[n]);
 
-      /* disconnect from all signals when we're finished */
-      g_signal_handlers_disconnect_matched (request->thumbnailers[n],
-                                            G_SIGNAL_MATCH_DATA,
-                                            0, 0, NULL, NULL, request);
+          /* tell the thumbnailer to generate the thumbnail */
+          tumbler_thumbnailer_create (lq->data, request->cancellables[n], request->infos[n]);
+
+          /* disconnect from all signals when we're finished */
+          g_signal_handlers_disconnect_by_data (lq->data, request);
+          g_signal_handlers_disconnect_by_data (lq->data, request->cancellables[n]);
+        }
     }
 
   /* free list */
@@ -518,6 +522,7 @@ tumbler_lifo_scheduler_thread (gpointer data,
 static void
 tumbler_lifo_scheduler_thumbnailer_error (TumblerThumbnailer      *thumbnailer,
                                           const gchar             *failed_uri,
+                                          GQuark                   error_domain,
                                           gint                     error_code,
                                           const gchar             *message,
                                           TumblerSchedulerRequest *request)
@@ -531,7 +536,7 @@ tumbler_lifo_scheduler_thumbnailer_error (TumblerThumbnailer      *thumbnailer,
 
   /* forward the error signal */
   g_signal_emit_by_name (request->scheduler, "error", request->handle, failed_uris, 
-                         error_code, message, request->origin);
+                         error_domain, error_code, message, request->origin);
 }
 
 
