@@ -43,6 +43,7 @@
 #include <tumblerd/tumbler-specialized-thumbnailer.h>
 #include <tumblerd/tumbler-utils.h>
 
+#define WARNING_MALFORMED_SECTION "Malformed section \"%s\" in file \"%s\": %s"
 
 /* Property identifiers */
 enum
@@ -91,10 +92,9 @@ static ThumbnailerInfo *thumbnailer_info_new              (void);
 static void             thumbnailer_info_free             (ThumbnailerInfo  *info);
 static void             thumbnailer_info_list_free        (gpointer          pointer);
 
-#ifdef DEBUG
+/* debug */
 static void             dump_overrides                    (TumblerManager   *manager);
 static void             dump_thumbnailers                 (TumblerManager   *manager);
-#endif
 
 
 
@@ -145,10 +145,10 @@ struct _OverrideInfo
 struct _ThumbnailerInfo
 {
   TumblerThumbnailer *thumbnailer;
-#ifdef DEBUG
-  GFile              *file;
-#endif
   gint                dir_index;
+
+  /* debug */
+  GFile              *file;
 };
 
 
@@ -248,8 +248,7 @@ tumbler_manager_finalize (GObject *object)
   g_list_free (manager->monitors);
 
   /* release all directory objects */
-  g_list_foreach (manager->directories, (GFunc) g_object_unref, NULL);
-  g_list_free (manager->directories);
+  g_list_free_full (manager->directories, g_object_unref);
 
   /* destroy the hash tables */
   g_hash_table_unref (manager->thumbnailers);
@@ -341,7 +340,11 @@ tumbler_manager_register_cb (TumblerExportedManager *skeleton,
   g_return_val_if_fail (TUMBLER_IS_MANAGER (manager), FALSE);
   g_return_val_if_fail (uri_schemes != NULL, FALSE);
   g_return_val_if_fail (mime_types != NULL, FALSE);
-  
+
+  g_debug ("Registering new specialized thumbnailer");
+  tumbler_util_dump_strv (G_LOG_DOMAIN, "Supported URI schemes", uri_schemes);
+  tumbler_util_dump_strv (G_LOG_DOMAIN, "Supported mime types", mime_types);
+
   sender_name = g_dbus_method_invocation_get_sender (invocation);
 
   tumbler_mutex_lock (manager->mutex);
@@ -524,20 +527,17 @@ tumbler_manager_parse_overrides (TumblerManager *manager,
   GList    *overrides = NULL;
   GFile    *directory;
   gchar   **sections;
-  gchar    *filename;
+  const gchar *filename;
   gchar    *uri_type;
   guint     n;
 
   g_return_val_if_fail (TUMBLER_IS_MANAGER (manager), NULL);
   g_return_val_if_fail (G_IS_FILE (file), NULL);
   
-  filename = g_file_get_path (file);
+  filename = g_file_peek_path (file);
 
   if (!g_file_test (filename, G_FILE_TEST_IS_REGULAR))
-    {
-      g_free (filename);
-      return NULL;
-    }
+    return NULL;
 
   /* allocate the key file */
   key_file = g_key_file_new ();
@@ -545,10 +545,9 @@ tumbler_manager_parse_overrides (TumblerManager *manager,
   /* try to load the key file from the overrides file */
   if (!g_key_file_load_from_file (key_file, filename, G_KEY_FILE_NONE, &error))
     {
-      g_warning (_("Failed to load the file \"%s\": %s"), filename, error->message);
+      g_warning (TUMBLER_WARNING_LOAD_FILE_FAILED, filename, error->message);
       g_clear_error (&error);
       g_key_file_free (key_file);
-      g_free (filename);
       return NULL;
     }
 
@@ -563,8 +562,7 @@ tumbler_manager_parse_overrides (TumblerManager *manager,
       info->name = g_key_file_get_string (key_file, sections[n], "Name", &error);
       if (info->name == NULL)
         {
-          g_warning (_("Malformed section \"%s\" in file \"%s\": %s"),
-                     sections[n], filename, error->message);
+          g_warning (WARNING_MALFORMED_SECTION, sections[n], filename, error->message);
           g_clear_error (&error);
 
           override_info_free (info);
@@ -577,8 +575,7 @@ tumbler_manager_parse_overrides (TumblerManager *manager,
                                                 "UriScheme", &error);
       if (info->uri_scheme == NULL)
         {
-          g_warning (_("Malformed section \"%s\" in file \"%s\": %s"),
-                     sections[n], filename, error->message);
+          g_warning (WARNING_MALFORMED_SECTION, sections[n], filename, error->message);
           g_clear_error (&error);
 
           override_info_free (info);
@@ -591,8 +588,7 @@ tumbler_manager_parse_overrides (TumblerManager *manager,
                                                "MimeType", &error);
       if (info->mime_type == NULL)
         {
-          g_warning (_("Malformed section \"%s\" in file \"%s\": %s"),
-                     sections[n], filename, error->message);
+          g_warning (WARNING_MALFORMED_SECTION, sections[n], filename, error->message);
           g_clear_error (&error);
 
           override_info_free (info);
@@ -604,9 +600,8 @@ tumbler_manager_parse_overrides (TumblerManager *manager,
       uri_type = g_strdup_printf ("%s-%s", info->uri_scheme, info->mime_type);
       if (g_strcmp0 (sections[n], uri_type) != 0)
         {
-          g_warning (_("Malformed section \"%s\" in file \"%s\": "
-                       "Mismatch between section name and UriScheme/MimeType"),
-                     sections[n], filename);
+          g_warning (WARNING_MALFORMED_SECTION, sections[n], filename,
+                     "Mismatch between section name and UriScheme/MimeType");
 
           g_free (uri_type);
           override_info_free (info);
@@ -627,7 +622,6 @@ tumbler_manager_parse_overrides (TumblerManager *manager,
 
   g_free (sections);
   g_key_file_free (key_file);
-  g_free (filename);
 
   return overrides;
 }
@@ -788,9 +782,7 @@ tumbler_manager_load_overrides (TumblerManager *manager)
       g_object_unref (file);
     }
 
-#ifdef DEBUG
   dump_overrides (manager);
-#endif
 }
 
 
@@ -932,7 +924,7 @@ tumbler_manager_load_thumbnailer (TumblerManager *manager,
   GList           *lp;
   gchar          **hash_keys;
   gchar           *base_name;
-  gchar           *filename;
+  const gchar     *filename;
   gchar           *name;
   gchar           *object_path;
   gchar          **uri_schemes;
@@ -943,7 +935,7 @@ tumbler_manager_load_thumbnailer (TumblerManager *manager,
   g_return_if_fail (G_IS_FILE (file));
 
   /* determine the absolute filename of the input file */
-  filename = g_file_get_path (file);
+  filename = g_file_peek_path (file);
 
   /* allocate a new key file object */
   key_file = g_key_file_new ();
@@ -951,11 +943,10 @@ tumbler_manager_load_thumbnailer (TumblerManager *manager,
   /* try to load the key file data from the input file */
   if (!g_key_file_load_from_file (key_file, filename, G_KEY_FILE_NONE, &error))
     {
-      g_warning (_("Failed to load the file \"%s\": %s"), filename, error->message);
+      g_warning (TUMBLER_WARNING_LOAD_FILE_FAILED, filename, error->message);
       g_clear_error (&error);
 
       g_key_file_free (key_file);
-      g_free (filename);
 
       return;
     }
@@ -964,11 +955,10 @@ tumbler_manager_load_thumbnailer (TumblerManager *manager,
   name = g_key_file_get_string (key_file, "Specialized Thumbnailer", "Name", &error);
   if (name == NULL)
     {
-      g_warning (_("Malformed file \"%s\": %s"), filename, error->message);
+      g_warning (TUMBLER_WARNING_MALFORMED_FILE, filename, error->message);
       g_clear_error (&error);
 
       g_key_file_free (key_file);
-      g_free (filename);
 
       return;
     }
@@ -978,11 +968,10 @@ tumbler_manager_load_thumbnailer (TumblerManager *manager,
                                        "ObjectPath", &error);
   if (object_path == NULL)
     {
-      g_warning (_("Malformed file \"%s\": %s"), filename, error->message);
+      g_warning (TUMBLER_WARNING_MALFORMED_FILE, filename, error->message);
       g_clear_error (&error);
 
       g_key_file_free (key_file);
-      g_free (filename);
 
       return;
     }
@@ -992,13 +981,12 @@ tumbler_manager_load_thumbnailer (TumblerManager *manager,
                                            "MimeTypes", NULL, &error);
   if (mime_types == NULL)
     {
-      g_warning (_("Malformed file \"%s\": %s"), filename, error->message);
+      g_warning (TUMBLER_WARNING_MALFORMED_FILE, filename, error->message);
       g_clear_error (&error);
 
       g_free (object_path);
       g_free (name);
       g_key_file_free (key_file);
-      g_free (filename);
 
       return;
     }
@@ -1018,23 +1006,21 @@ tumbler_manager_load_thumbnailer (TumblerManager *manager,
   /* determine the time the file was last modified */
   if (g_stat (filename, &file_stat) != 0)
     {
-      g_warning (_("Failed to determine last modified time of \"%s\""), filename); 
+      g_warning ("Failed to determine last modified time of \"%s\"", filename);
 
       g_strfreev (uri_schemes);
       g_strfreev (mime_types);
       g_free (object_path);
       g_free (name);
       g_key_file_free (key_file);
-      g_free (filename);
 
       return;
     }
 
   /* allocate a new thumbnailer info */
   info = thumbnailer_info_new ();
-#ifdef DEBUG
-  info->file = g_object_ref (file);
-#endif
+  if (tumbler_util_is_debug_logging_enabled (G_LOG_DOMAIN))
+    info->file = g_object_ref (file);
 
   /* compute the directory index of the input file */
   directory = g_file_get_parent (file);
@@ -1055,7 +1041,6 @@ tumbler_manager_load_thumbnailer (TumblerManager *manager,
   g_free (object_path);
   g_free (name);
   g_key_file_free (key_file);
-  g_free (filename);
   
   /* determine the basename of the file */
   base_name = g_file_get_basename (file);
@@ -1180,22 +1165,15 @@ tumbler_manager_load_thumbnailers (TumblerManager *manager,
   const gchar *base_name;
   GFileType    type;
   GFile       *file;
-  gchar       *dirname;
   GDir        *dir;
 
   g_return_if_fail (TUMBLER_IS_MANAGER (manager));
   g_return_if_fail (G_IS_FILE (directory));
 
-  /* determine the absolute path to the directory */
-  dirname = g_file_get_path (directory);
-
   /* try to open the directory for reading */
-  dir = g_dir_open (dirname, 0, NULL);
+  dir = g_dir_open (g_file_peek_path (directory), 0, NULL);
   if (dir == NULL)
-    {
-      g_free (dirname);
-      return;
-    }
+    return;
 
   /* iterate over all files in the directory */
   for (base_name = g_dir_read_name (dir); 
@@ -1220,9 +1198,6 @@ tumbler_manager_load_thumbnailers (TumblerManager *manager,
 
   /* close the directory handle */
   g_dir_close (dir);
-
-  /* free memory used for the directory path */
-  g_free (dirname);
 }
 
 
@@ -1238,13 +1213,8 @@ tumbler_manager_load_thumbnailers (TumblerManager *manager,
 void
 tumbler_manager_load (TumblerManager *manager)
 {
-  GHashTable         *single_path;
-  const gchar *const *data_dirs;
-  GFileMonitor       *monitor;
-  GList              *directories = NULL;
-  GList              *iter;
-  gchar              *dirname;
-  guint               n;
+  GFileMonitor *monitor;
+  GList *directories, *iter;
 
   g_return_if_fail (TUMBLER_MANAGER (manager));
 
@@ -1256,47 +1226,7 @@ tumbler_manager_load (TumblerManager *manager)
 
   tumbler_mutex_unlock (manager->mutex);
 
-  /* prepend $XDG_DATA_HOME/thumbnailers/ to the directory list */
-  dirname = g_build_filename (g_get_user_data_dir (), "thumbnailers", NULL);
-  directories = g_list_prepend (directories, g_file_new_for_path (dirname));
-  g_free (dirname);
-
-  /* determine system data dirs */
-  data_dirs = g_get_system_data_dirs ();
-
-  /* Create a ghash table to insert loaded directory path to avoid duplication */
-  single_path = g_hash_table_new_full (g_file_hash, (GEqualFunc)g_file_equal, g_object_unref, NULL);
-
-  /* build $XDG_DATA_DIRS/thumbnailers dirnames and prepend them to the list */
-  for (n = 0; data_dirs[n] != NULL; ++n)
-    {
-      GFile *path;
-
-      path = g_file_new_for_path(data_dirs[n]);
-
-      if (!g_hash_table_lookup (single_path, path))
-        {
-          /* Save it in the hash table so we can relocate it */
-          /* path will be free automatically by g_hash_table_destroy */
-          g_hash_table_insert (single_path, path, path);
-
-          dirname = g_build_filename (data_dirs[n], "thumbnailers", NULL);
-          directories = g_list_prepend (directories, g_file_new_for_path (dirname));
-          g_free (dirname);
-        }
-      else
-        {
-          /* Free the path GFile object */
-          g_object_unref(path);
-        }
-    }
-
-  /* destroy the hash table used for loading single pathes */
-  g_hash_table_destroy (single_path);
-
-  /* reverse the directory list so that the directories with highest 
-   * priority come first */
-  directories = g_list_reverse (directories);
+  directories = tumbler_util_get_thumbnailer_dirs ();
 
   tumbler_mutex_lock (manager->mutex);
 
@@ -1308,9 +1238,7 @@ tumbler_manager_load (TumblerManager *manager)
   for (iter = manager->directories; iter != NULL; iter = iter->next)
     tumbler_manager_load_thumbnailers (manager, iter->data);
 
-#ifdef DEBUG
   dump_thumbnailers (manager);
-#endif
 
   /* update the overrides cache */
   tumbler_manager_load_overrides (manager);
@@ -1636,39 +1564,31 @@ tumbler_manager_directory_changed (TumblerManager   *manager,
 
   if (event_type == G_FILE_MONITOR_EVENT_DELETED)
     {
-      base_name = g_file_get_basename (file);
+      g_debug ("  %s deleted", g_file_peek_path (file));
 
+      base_name = g_file_get_basename (file);
       if (g_strcmp0 (base_name, "overrides") == 0)
         {
-#ifdef DEBUG
-          g_debug ("  %s deleted", g_file_get_path (file));
-#endif
           tumbler_mutex_lock (manager->mutex);
           tumbler_manager_unload_overrides_file (manager, file);
           tumbler_registry_update_supported (manager->registry);
-#ifdef DEBUG
+
           dump_overrides (manager);
-#endif
+
           tumbler_mutex_unlock (manager->mutex);
         }
       else if (g_str_has_suffix (base_name, ".service"))
         {
-#ifdef DEBUG
-          g_debug ("  %s deleted", g_file_get_path (file));
-#endif
           tumbler_mutex_lock (manager->mutex);
           tumbler_manager_thumbnailer_file_deleted (manager, file);
           tumbler_registry_update_supported (manager->registry);
-#ifdef DEBUG
+
           dump_thumbnailers (manager);
-#endif
+
           tumbler_mutex_unlock (manager->mutex);
         }
       else
         {
-#ifdef DEBUG
-          g_debug ("  %s deleted", g_file_get_path (file));
-#endif
           tumbler_mutex_lock (manager->mutex);
           dir_index = tumbler_manager_get_dir_index (manager, file);
           tumbler_mutex_unlock (manager->mutex);
@@ -1678,10 +1598,10 @@ tumbler_manager_directory_changed (TumblerManager   *manager,
               tumbler_mutex_lock (manager->mutex);
               tumbler_manager_directory_deleted (manager, file, dir_index);
               tumbler_registry_update_supported (manager->registry);
-#ifdef DEBUG
+
               dump_overrides (manager);
               dump_thumbnailers (manager);
-#endif
+
               tumbler_mutex_unlock (manager->mutex);
             }
         }
@@ -1698,29 +1618,27 @@ tumbler_manager_directory_changed (TumblerManager   *manager,
             {
               if (event_type == G_FILE_MONITOR_EVENT_CREATED)
                 {
-#ifdef DEBUG
-                  g_debug ("  %s created", g_file_get_path (file));
-#endif
+                  g_debug ("  %s created", g_file_peek_path (file));
+
                   tumbler_mutex_lock (manager->mutex);
                   tumbler_manager_load_overrides_file (manager, file);
                   tumbler_registry_update_supported (manager->registry);
-#ifdef DEBUG
+
                   dump_overrides (manager);
-#endif
+
                   tumbler_mutex_unlock (manager->mutex);
                 }
               else if (event_type == G_FILE_MONITOR_EVENT_CHANGES_DONE_HINT)
                 {
-#ifdef DEBUG
-                  g_debug ("  %s changed", g_file_get_path (file));
-#endif
+                  g_debug ("  %s changed", g_file_peek_path (file));
+
                   tumbler_mutex_lock (manager->mutex);
                   tumbler_manager_unload_overrides_file (manager, file);
                   tumbler_manager_load_overrides_file (manager, file);
                   tumbler_registry_update_supported (manager->registry);
-#ifdef DEBUG
+
                   dump_overrides (manager);
-#endif
+
                   tumbler_mutex_unlock (manager->mutex);
                 }
             }
@@ -1728,29 +1646,27 @@ tumbler_manager_directory_changed (TumblerManager   *manager,
             {
               if (event_type == G_FILE_MONITOR_EVENT_CREATED)
                 {
-#ifdef DEBUG
-                  g_debug ("  %s created", g_file_get_path (file));
-#endif
+                  g_debug ("  %s created", g_file_peek_path (file));
+
                   tumbler_mutex_lock (manager->mutex);
                   tumbler_manager_load_thumbnailer (manager, file);
                   tumbler_registry_update_supported (manager->registry);
-#ifdef DEBUG
+
                   dump_thumbnailers (manager);
-#endif
+
                   tumbler_mutex_unlock (manager->mutex);
                 }
               else if (event_type == G_FILE_MONITOR_EVENT_CHANGES_DONE_HINT)
                 {
-#ifdef DEBUG
-                  g_debug ("  %s changed", g_file_get_path (file));
-#endif
+                  g_debug ("  %s changed", g_file_peek_path (file));
+
                   tumbler_mutex_lock (manager->mutex);
                   tumbler_manager_thumbnailer_file_deleted (manager, file);
                   tumbler_manager_load_thumbnailer (manager, file);
                   tumbler_registry_update_supported (manager->registry);
-#ifdef DEBUG
+
                   dump_thumbnailers (manager);
-#endif
+
                   tumbler_mutex_unlock (manager->mutex);
                 }
             }
@@ -1763,17 +1679,15 @@ tumbler_manager_directory_changed (TumblerManager   *manager,
 
           if (dir_index >= 0)
             {
-#ifdef DEBUG
-              g_debug ("  %s created", g_file_get_path (file));
-#endif
+              g_debug ("  %s created", g_file_peek_path (file));
 
               tumbler_mutex_lock (manager->mutex);
               tumbler_manager_directory_created (manager, file, dir_index);
               tumbler_registry_update_supported (manager->registry);
-#ifdef DEBUG
+
               dump_overrides (manager);
               dump_thumbnailers (manager);
-#endif
+
               tumbler_mutex_unlock (manager->mutex);
             }
         }
@@ -1836,9 +1750,9 @@ thumbnailer_info_free (ThumbnailerInfo *info)
   if (info == NULL)
     return;
 
-#ifdef DEBUG
-  g_object_unref (info->file);
-#endif
+  if (tumbler_util_is_debug_logging_enabled (G_LOG_DOMAIN))
+    g_object_unref (info->file);
+
   g_object_unref (info->thumbnailer);
   g_slice_free (ThumbnailerInfo, info);
 }
@@ -1860,7 +1774,6 @@ thumbnailer_info_list_free (gpointer pointer)
 
 
 
-#ifdef DEBUG
 static void
 dump_overrides (TumblerManager *manager)
 {
@@ -1868,18 +1781,27 @@ dump_overrides (TumblerManager *manager)
   const gchar   *hash_key;
   GList        **list;
   GList         *iter;
+  GString       *string;
 
-  g_print ("Overrides:\n");
+  if (! tumbler_util_is_debug_logging_enabled (G_LOG_DOMAIN))
+    return;
+
+  if (g_hash_table_size (manager->overrides) == 0)
+    return;
+
+  string = g_string_new ("Specialized thumbnailer overrides:\n");
 
   g_hash_table_iter_init (&table_iter, manager->overrides);
   while (g_hash_table_iter_next (&table_iter, (gpointer) &hash_key, (gpointer) &list))
     {
-      g_print ("  %s:\n", hash_key);
+      g_string_append_printf (string, "  %s:\n", hash_key);
       for (iter = *list; iter != NULL; iter = iter->next)
-        g_print ("    %s\n", ((OverrideInfo *)iter->data)->name);
+        g_string_append_printf (string, "    %s\n", ((OverrideInfo *) iter->data)->name);
     }
 
-  g_print ("\n");
+  g_string_truncate (string, string->len - 1);
+  g_debug ("%s", string->str);
+  g_string_free (string, TRUE);
 }
 
 
@@ -1891,20 +1813,29 @@ dump_thumbnailers (TumblerManager *manager)
   const gchar   *base_name;
   GList        **list;
   GList         *iter;
+  GString       *string;
 
-  g_print ("Thumbnailers:\n");
+  if (! tumbler_util_is_debug_logging_enabled (G_LOG_DOMAIN))
+    return;
+
+  if (g_hash_table_size (manager->thumbnailers) == 0)
+    return;
+
+  string = g_string_new ("Specialized thumbnailers:\n");
 
   g_hash_table_iter_init (&table_iter, manager->thumbnailers);
   while (g_hash_table_iter_next (&table_iter, (gpointer) &base_name, (gpointer) &list))
     {
-      g_print ("  %s:\n", base_name);
+      g_string_append_printf (string, "  %s:\n", base_name);
       for (iter = *list; iter != NULL; iter = iter->next)
-        g_print ("    %s\n", g_file_get_path (((ThumbnailerInfo *)iter->data)->file));
+        g_string_append_printf (string, "    %s\n",
+                                g_file_peek_path (((ThumbnailerInfo *) iter->data)->file));
     }
 
-  g_print ("\n");
+  g_string_truncate (string, string->len - 1);
+  g_debug ("%s", string->str);
+  g_string_free (string, TRUE);
 }
-#endif /* DEBUG */
 
 
 
