@@ -95,6 +95,7 @@ static gboolean tumbler_service_get_flavors_cb  (TumblerExportedService  *skelet
 static void tumbler_service_scheduler_error    (TumblerScheduler   *scheduler,
                                                 guint32             handle,
                                                 const gchar *const *failed_uris,
+                                                GQuark              error_domain,
                                                 gint                error_code,
                                                 const gchar        *message,
                                                 const gchar        *origin,
@@ -382,6 +383,7 @@ tumbler_service_set_property (GObject      *object,
 static gboolean
 tumbler_service_error_idle (gpointer user_data)
 {
+  TumblerCache      *cache;
   SchedulerIdleInfo *info = user_data;
   GVariant          *signal_variant;
 
@@ -392,6 +394,11 @@ tumbler_service_error_idle (gpointer user_data)
   g_return_val_if_fail (info->origin != NULL && *info->origin != '\0', FALSE);
   g_return_val_if_fail (TUMBLER_IS_SERVICE (info->service), FALSE);
 
+  /* cache cleanup: any previous thumbnail is now invalid */
+  cache = tumbler_cache_get_default ();
+  tumbler_cache_delete (cache, (const gchar *const *) info->uris);
+  g_object_unref (cache);
+
   /* signal variant */
   signal_variant = g_variant_new ("(u^asis)",
                                   info->handle,
@@ -419,6 +426,7 @@ static void
 tumbler_service_scheduler_error (TumblerScheduler   *scheduler,
                                  guint32             handle,
                                  const gchar *const *failed_uris,
+                                 GQuark              error_domain,
                                  gint                error_code,
                                  const gchar        *message,
                                  const gchar        *origin,
@@ -431,16 +439,34 @@ tumbler_service_scheduler_error (TumblerScheduler   *scheduler,
   g_return_if_fail (message != NULL && *message != '\0');
   g_return_if_fail (origin != NULL && *origin != '\0');
   g_return_if_fail (TUMBLER_IS_SERVICE (service));
-  
+
+  /* request cancelled at some level after being handled by the thumbnailer: no signal,
+   * just as if it had been cancelled before */
+  if (error_domain == G_IO_ERROR && error_code == G_IO_ERROR_CANCELLED)
+    return;
+
   info = g_slice_new0 (SchedulerIdleInfo);
 
   info->scheduler = g_object_ref (scheduler);
   info->handle = handle;
   info->uris = g_strdupv ((gchar **)failed_uris);
-  info->error_code = error_code;
-  info->message = g_strdup (message);
   info->origin = g_strdup (origin);
   info->service = g_object_ref (service);
+  if (error_domain == TUMBLER_ERROR)
+    {
+      info->error_code = error_code;
+      info->message = g_strdup (message);
+    }
+  else
+    {
+      info->error_code = TUMBLER_ERROR_OTHER_ERROR_DOMAIN;
+      info->message = g_strdup_printf ("(domain %s, code %d) %s",
+                                       g_quark_to_string (error_domain), error_code, message);
+    }
+
+  g_debug ("Error signal for job %d: Code %d, message: %s",
+           handle, info->error_code, info->message);
+  tumbler_util_dump_strv (G_LOG_DOMAIN, "URIs", failed_uris);
 
   g_idle_add (tumbler_service_error_idle, info);
 }
@@ -491,7 +517,9 @@ tumbler_service_scheduler_finished (TumblerScheduler *scheduler,
   g_return_if_fail (TUMBLER_IS_SCHEDULER (scheduler));
   g_return_if_fail (origin != NULL && *origin != '\0');
   g_return_if_fail (TUMBLER_IS_SERVICE (service));
-  
+
+  g_debug ("Finishing job %d\n", handle);
+
   info = g_slice_new0 (SchedulerIdleInfo);
 
   info->scheduler = g_object_ref (scheduler);
@@ -549,7 +577,10 @@ tumbler_service_scheduler_ready (TumblerScheduler   *scheduler,
   g_return_if_fail (origin != NULL && *origin != '\0');
   g_return_if_fail (uris != NULL && uris[0] != NULL && *uris[0] != '\0');
   g_return_if_fail (TUMBLER_IS_SERVICE (service));
-  
+
+  g_debug ("Ready signal for job %d", handle);
+  tumbler_util_dump_strv (G_LOG_DOMAIN, "URIs", uris);
+
   info = g_slice_new0 (SchedulerIdleInfo);
 
   info->scheduler = g_object_ref (scheduler);
@@ -604,6 +635,8 @@ tumbler_service_scheduler_started (TumblerScheduler *scheduler,
   g_return_if_fail (origin != NULL && *origin != '\0');
   g_return_if_fail (TUMBLER_IS_SERVICE (service));
 
+  g_debug ("Starting job %d", handle);
+
   info = g_slice_new0 (SchedulerIdleInfo);
 
   info->scheduler = g_object_ref (scheduler);
@@ -683,10 +716,10 @@ tumbler_service_queue_cb (TumblerExportedService  *skeleton,
 {
   TumblerSchedulerRequest *scheduler_request;
   TumblerThumbnailFlavor  *flavor;
-  TumblerThumbnailer     **thumbnailers;
   TumblerScheduler        *scheduler = NULL;
   TumblerFileInfo        **infos;
   TumblerCache            *cache;
+  GList                  **thumbnailers;
   GList                   *iter;
   gchar                   *name;
   const gchar             *origin;
@@ -731,6 +764,9 @@ tumbler_service_queue_cb (TumblerExportedService  *skeleton,
   /* get the request handle */
   handle = scheduler_request->handle;
 
+  g_debug ("Handling request %d", handle);
+  tumbler_util_dump_strvs_side_by_side (G_LOG_DOMAIN, "URIs", "Mime types", uris, mime_hints);
+
   /* iterate over all schedulers */
   for (iter = service->schedulers; iter != NULL; iter = iter->next)
     {
@@ -752,7 +788,7 @@ tumbler_service_queue_cb (TumblerExportedService  *skeleton,
 
   /* default to the first scheduler in the list if we couldn't find
    * the scheduler with the desired name */
-  if (scheduler == NULL) 
+  if (scheduler == NULL && service->schedulers != NULL)
     scheduler = TUMBLER_SCHEDULER (service->schedulers->data);
 
   /* report unsupported flavors back to the client */
@@ -763,9 +799,9 @@ tumbler_service_queue_cb (TumblerExportedService  *skeleton,
                                          service);
 
       /* emit an error signal */
-      tumbler_service_scheduler_error (scheduler, handle, uris, 
+      tumbler_service_scheduler_error (scheduler, handle, uris, TUMBLER_ERROR,
                                        TUMBLER_ERROR_UNSUPPORTED_FLAVOR, 
-                                       _("Unsupported thumbnail flavor requested"),
+                                       TUMBLER_ERROR_MESSAGE_UNSUPPORTED_FLAVOR,
                                        scheduler_request->origin,
                                        service);
 
@@ -809,6 +845,8 @@ tumbler_service_dequeue_cb (TumblerExportedService  *skeleton,
 
   if (handle != 0) 
     {
+      g_debug ("Dequeuing files for job %d", handle);
+
       /* iterate over all available schedulers */
       for (iter = service->schedulers; iter != NULL; iter = iter->next)
         {
@@ -928,10 +966,7 @@ tumbler_service_get_flavors_cb  (TumblerExportedService  *skeleton,
       tumbler_exported_service_complete_get_flavors (skeleton, invocation, flavor_strings);
       
       g_free (flavor_strings);
-
-      g_list_foreach (flavors, (GFunc) g_object_unref, NULL);
-      g_list_free (flavors);
-
+      g_list_free_full (flavors, g_object_unref);
       g_object_unref (cache);
     }
   else
